%read from file.
format short;
clc
gpsCoordinates = 'data_GPSext.dat';      % File must be in same directory
model = 'data_GGM02S_110.dat';         % File must be in same directory

fid = fopen(gpsCoordinates);                      % Open this file 
    C = textscan(fid, '%f %f');
    long=C{2};             % Fetch longitude
    lat=C{1};              % Fetch latitude
    w = length(long);      % Number of angle pairs
    angleTable = zeros(w,2);
    
fid2 = fopen(model);
    D = textscan(fid2, '%f %f %f %f');
    nFile=D{1};             % Fetch n
    mFile=D{2};             % Fetch m
    jnmFile=D{3};           % Fetch j_nm
    knmFile=D{4};           % Fetch k_nm

    nMax = nFile(length(nFile)); 
    mMax = nMax;            % Set max values for n and m

    a = 6378137;            % Major axis            [meters]
    b = 6356752.3;          % Minor axis            [meters]
    f = 298.257222101;      % Actually f^-1
    etwo = 0.006694380023;  % Eccentricity e^2
    e = sqrt(etwo);         % Eccentricity
    Radius = 6371000;       % Earth mean radius     [meters]
    GM = 3986004.415*10^8;  % Gravitational constant[m^3*kg^-1*s^-2]
    gamma = 9.81;           % Grav. acceleration    [m*s^-2]
    JtwoN = 0.108263*10^-2; % For GRS80
    JtwoNHat = -JtwoN/sqrt(5);
    constOutside = GM/(Radius*gamma);
    P = zeros(w,w);
    c = 0;
        
    for i = 0:w-1
        geomTable(1+(i),1) = long(1+(i));
        geomTable(1+(i),2) = lat(1+(i));
        angleTable(1+(i),1) = deg2rad(geomTable(1+(i),1));
        angleTable(1+(i),2) = deg2rad(geomTable(1+(i),2));
    end
    
    Jn = zeros(nMax+1,1);   % Instansiate Jn array
    Jn(1+(1)) = 0;

    P = zeros(nMax,mMax);   % Initiate R, P and q matrix. Set to zero.
    R = zeros(nMax,mMax);
    q = zeros(nMax,mMax);   %% End of INIT.

if mod(nMax,2) == 0
    for n = 2:2:nMax
        Jn(n) = 0;
        Jn(n+1) = (-1)^(n/2)*((3*(e^n))*(1-(n/2)+(5/2)*(JtwoN/(etwo))*n))/((n+1)*(n+3)*(sqrt(2*n+1)));
    end
else
    for n = 2:2:nMax-1
        Jn(n) = 0;
        Jn(n+1) = (-1)^(n/2)*((3*(e^n))*(1-(n/2)+(5/2)*(JtwoN/(etwo))*n))/((n+1)*(n+3)*(sqrt(2*n+1)));
    end
end  % Assign Jn based on an Odd or Even number of nMax

c = 0;                  % Var used for counting
for n = 2:nMax          % Calculate R-matrix
    R(1+(n),1+(0)) = jnmFile(1+(c)) - Jn(1+(n));
    for m = 1:n
        c = c + 1;
        R(1+(n),1+(m)) = jnmFile(1+(c));
    end
c = c + 1;
end    

c = 0;                  % Var used for counting
for n = 2:nMax          % Calculate q-matrix
    for m = 0:n
        c = c + 1;
        q(1+(n),1+(m)) = knmFile(c);
    end
end                     %% End of J_n, R_nm and q_nm
    
    c = 0;
    for phi = 0:1:w-1       % Phi moves through input values
        c = c + 1;
        clc;
        100*c/w
       
        phiCurrent = angleTable(1+(phi),2);
        %phiInDegree = rad2deg(phiCurrent);
        %Radius = sqrt(a^2 * cos(phiInDegree)^2 + b^2 * sin(phiInDegree)^2);
        %Radius
        
        %following lines calculate and display completeness
        t = sin(phiCurrent);% Now insert initial P parameters
        P(1+(0),1+(0)) = 1; 
        P(1+(1),1+(0)) = t*sqrt(3); 
        P(1+(1),1+(1)) = sqrt(3)*sqrt(1-t^2);

        for n = 2:nMax      % Calculate rest of P-matrix
            for m = 0:n
                %First rule
                if (n>=2 && m==0)
                    P(1+(n),1+(0)) = -(sqrt(2*n+1)/n)*((n-1)/sqrt(2*n-3))*P(1+(n-2),1+(0))+t*(sqrt(2*n+1)/n)*sqrt(2*n-1)*P(1+(n-1),1+(0));
                end

                %Second rule
                if (n>=2 && m==n)
                    P(1+(n),1+(n)) = sqrt((2*n+1)/(2*n))*sqrt(1-t^2)*P(1+(n-1),1+(n-1));
                end

                %Third rule
                if (n>=1 && m==(n-1))
                    P(1+(n),1+(n-1)) = t*sqrt(2*n+1)*P(1+(n-1),1+(n-1));          
                end

                %Fourh rule
                if (n>=3 && 1<=m && m<=(n-2))
                    P(1+(n),1+(m)) = -sqrt(((2*n+1)*(n+m-1)*(n-m-1))/((2*n-3)*(n+m)*(n-m)))*P(1+(n-2),1+(m))+t*sqrt(((2*n+1)*(2*n-1))/((n+m)*(n-m)))*P(1+(n-1),1+(m));
                end
            end
        end

        for lamda = 0:1:w-1 %For each longitude 
            lamdaCurrent = angleTable(1+(phi),1);
            Ntot = 0;               % Reset N for new angle pair
            nSum = 0;               % Reset sum n
            mSumOne = 0;

            for n = 2:nMax
            mSum = 0;               % Reset sum m

                for m = 0:n
                    mSumOne = (R(1+(n),1+(m))*cos(m*lamdaCurrent)+q(1+(n),1+(m))*sin(m*lamdaCurrent))* P(1+(n),1+(m));
                    mSum = mSum + mSumOne;
                end

                mSum = mSum * (a/Radius)^(n);
                Ntot = Ntot + mSum;
            end
            Ntot = Ntot * constOutside;
            geomTable(1+(phi),3) = Ntot;
            %geomTable(1+(phi),4) = N8(1+(phi),3);
            %geomTable(1+(phi),5) = geomTable(1+(phi),3)-geomTable(1+(phi),4);
          
        end
    end
    
    disp(' ');
    disp('Results:');
    disp('*Difference is Ngrav - Ngeom');
    disp('     long      lat      Ngrav     Ngeom   Difference*');
    disp(geomTable);