%-----------{ Notes  }----------%
%                               %
% GGM03S uses order/degree 130  %
% GGM03C uses order/degree 360  %
% GGM02S uses order/degree 200  %
% GGM02C uses order/degree 110  %
% Radius in ALL GGM = 6371km    %
%                               %        
%                               %
%-------------------------------%


%-----------{ Step 1 }----------%
%  Select model to draw in map  %
%                               %
%        % 1 = GGM02C           %
%        % 2 = GGM03C           %
%        % 3 = GGM02C_corrected % 
%        % 4 = GGM03C_corrected %
%        % 5 = EGM96            % 
%        % 6 = EGM08            %
%        % 7 = GRID 12          %
%        % 8 = GGM02 vs GGM03   %
%        % 9 = Grid 13          %
%        %10 = GGM03S           %
%        %11 = GGM03S_corrected %
%        %12 = GGM02S           %
%        %13 = GGM02S_corrected %
%        %14 = Corrections only %
%                               %
compareModel = 7;               %
%-------------------------------%

delete(figure);
load('MyColormaps','mycmap')
selectedProjection = 'Lambert';

%clc;                % Clear screen
format short;       % Fewer visible digits

% Read from file
% File must be in same directory
filename = 'lat_lng_ggm02C_ggm03C_egm96_egm08_gps_corr_grid12_grid13_ggm03S_ggm02S.dat';      
fid = fopen(filename);          % Open this file 
C = textscan(fid, ' %f %f %f %f %f %f %f %f %f %f %f %f');
lat=C{1};           % Latitude
lng=C{2};           % Longitude
H_GGM02C=C{3};      % GGM02C-height 
H_GGM03C=C{4};      % GGM03C-height
H_egm96=C{5};       % EGM96-height
H_egm08=C{6};       % EGM08-height
H_gps=C{7};         % GPS derived geoid height
GGM_corr=C{8};      % Corrections to GGM model
H_grid12 = C{9};    % Grid heights
H_grid13 = C{10};   % Grid 13
H_GGM03S = C{11};   % GGM03S
H_GGM02S = C{12};   % GGM02S
% Initiate tables
n = length(lat);    % Set size of iteration table
N = zeros(n,11);    % Table of all the data
Z = zeros(n,3);     % Table used in plot

% Fill out this table
correctedGGM02C = zeros(n,1);
correctedGGM03C = zeros(n,1);
correctedGGM02S = zeros(n,1);
correctedGGM03S = zeros(n,1);

N(:,1) = lat(:);                % Latitude
N(:,2) = lng(:);                % Longitude
N(:,3) = H_GGM02C(:);           % GGM02C
N(:,4) = H_GGM03C(:);           % GGM03C
N(:,5) = H_egm96(:);            % EGM96
N(:,6) = H_egm08(:);            % EGM08
N(:,7) = H_gps(:);              % GPS heights
N(:,8) = H_grid12(:);           % Grid 12 heigths
N(:,9) = H_grid13(:);           % Grid 13 heights
N(:,10) = H_GGM03S(:);          % GGM03S
N(:,11) = H_GGM02S(:);          % GGM02S
%-------------------------------------------------

correctedGGM02C(:) = N(:,3) + GGM_corr(:);
correctedGGM03C(:) = N(:,4) + GGM_corr(:);
correctedGGM02S(:) = N(:,11) + GGM_corr(:);
correctedGGM03S(:) = N(:,10) + GGM_corr(:);
%-------------------------------------------------

N(:,12) = N(:,7) - N(:,3);            % GPS minus GGM02C
N(:,13) = N(:,7) - N(:,4);            % GPS minus GGM03C
N(:,14) = N(:,7) - correctedGGM02C(:);% GPS minus Corrected GGM02
N(:,15) = N(:,7) - correctedGGM03C(:);% GPS minus Corrected GGM03
N(:,16) = N(:,7) - N(:,5);            % GPS minus EGM96
N(:,17) = N(:,7) - N(:,6);            % GPS minus EGM08
N(:,18) = N(:,7) - N(:,8);            % GPS minus Grid12
N(:,19) = N(:,3) - N(:,4);            % GGM02 minus GGM03
N(:,20) = N(:,9);                     % GPS minus Grid13
N(:,21) = N(:,7) - N(:,10);           % GPS minus GGM03S
N(:,22) = N(:,7) - correctedGGM03S(:);% GPS minus Corrected GGM03s
N(:,23) = N(:,7) - N(:,11);           % GPS minus GGM02S
N(:,24) = N(:,7) - correctedGGM02S(:);% GPS minus Corrected GGM03s
N(:,25) = N(:,8) - N(:,7);            % Korreksjoner



% Get to the right column in the N matrix; clear screen
compareModel = compareModel + 11; cla; clf;

% Seacolor. Recommended values from ESRI.
seaColor = [.816 .910 .996];

% Create initial figure
fig('units','centimeters','width',10,'height',9.2,'font','Helvetica','fontsize',11)
axesm('MapProjection', selectedProjection ,'MapLatLimit',[57 72], 'MapLonLimit',[0 35], 'grid', 'on'); framem; gridm; 

[Z(:,1),Z(:,2)] = mfwdtran(N(:,1),N(:,2));
set(gca,'Ylim',[0.84 1.2]);
set(gca, 'visible', 'off');
T = get(gca,'tightinset');
set(gca,'position',[T(1) T(2) 1-T(1)-T(3) 1-T(2)-T(4)]);
geoshow('ne_50m_ocean.shp','FaceColor',seaColor, 'EdgeColor', 'black');

delete(figure(1))

% Set the cosmetics of the map
mlabel on; plabel on; gridm on;
setm(gca,'MLabelLocation',[0 10 20 30])
setm(gca,'MLineLocation',[0 10 20 30])
setm(gca,'PLabelLocation',[60 65  70])
setm(gca,'PLineLocation',[60 65  70])

%Draw scatter plot
Z(:,3) = N(:,compareModel(:));
scatter(Z(:,1),Z(:,2),9,Z(:,3),'filled');

%------- Adjust axis coloring --------
graphLimit = max(abs(max(N(:,compareModel))),abs(min(N(:,compareModel))));

v = caxis;
v(1) = -1*graphLimit;
v(2) = graphLimit;

if min(N(:,compareModel)) > 0
    v(1) = 0;
    caxis([v(1) v(2)]);
    colormap jet;
    colorbar south;
elseif max(N(:,compareModel)) > 0
    set(gcf,'Colormap',mycmap)
    colorbar south;
    caxis([v(1) v(2)]);
else 
    v(2) = 0;
    caxis([v(1) v(2)]);
    colormap(flipud(jet));
    colorbar south;
end
%-------------------------------------

%--------- Display key data ----------
format long;
disp('std,mean,max,min');
disp(std(N(:,compareModel)));
disp(mean(N(:,compareModel)));
disp(max(N(:,compareModel)));
disp(min(N(:,compareModel)));
disp('-----------------------');
%-------------------------------------